"""
Telecom Systems Development Kit - tsdk
"""
import io

import pandas as pd

mapping = [
    ['B1', 'L1900C1', 'MB'],
    ['B2', 'L1900C2', 'MB'],
    ['D1', 'L700', 'LB'],
    ['E1', 'L600', 'LB'],
    ['L1', 'L2100', 'MB'],
    ['K1', 'NR600', 'LB'],
    ['T1', 'L2500C1', 'Anchor'],
    ['T2', 'L2500C2', 'Anchor'],
    ['A1', 'NR2500C1', 'Anchor'],
    ['A2', 'NR2500C2', 'Anchor']
]


def find_mapping(sl, f):
    r = None
    for i, e in enumerate(sl):
        try:
            test = e.index(f)
            r = i
            assert test
            break
        except ValueError:
            pass
    return r


def parse_cmd_results(cmd_results):
    # Total: 3 MOs attempted, 3 MOs set
    import numpy
    x = list(
        map(lambda a: [int(a.split(' ')[1].strip()),
                       int(a.split(' ')[4].strip())] if 'Total: ' in a else [0, 0], cmd_results))
    npa = numpy.array(x)
    return npa.sum(axis=0).tolist()


def parse_st_cell(st_cell_results):
    return sorted(
        set(
            filter(
                lambda a: a is not None,
                map(
                    lambda a: (
                                  a.split('=')[2]
                              ).strip()[:1] + (
                                                  a.split('=')[2]
                                              ).strip()[-1:] if len(
                        a.split('=')
                    ) == 3 else None,
                    st_cell_results
                )
            )
        )
    )


def parse_nodes_list(nodes_list_results):
    return sorted(set(map(lambda a: (a.split(' ')[1]).strip(), nodes_list_results)))


class Sshc(object):
    @staticmethod
    def run_cmd(s):
        return [s]

    @staticmethod
    def run_cmd_file(s):
        return [s]


ssh = Sshc()

cmd_path = '/location/'
sites_list = ['NJ01059A', '', '', '']
final_result = ['Site,Node,MOs Attempted,MOs Set']
for site in sites_list:
    nodes_list = ssh.run_cmd(
        'cat /opt/ericsson/amos/moshell/sitefiles/ipdatabase | grep -i -h %s' % site)
    nodes_list = parse_nodes_list(nodes_list)
    for node in nodes_list:
        ssh.run_cmd('amos %s' % node)
        ssh.run_cmd('lt all')
        results = ssh.run_cmd('st cell')
        results = parse_st_cell(results)
        for result in results:
            _, cmd_file_a, cmd_file_b = mapping[find_mapping(mapping, result)]
            result_a = ssh.run_cmd_file(cmd_path + cmd_file_a)
            result_b = ssh.run_cmd_file(cmd_path + cmd_file_b)
            final_result.append('%s,%s,%s' % (site, node,
                                              ','.join(list(map(str, parse_cmd_results(result_a))))))
            final_result.append('%s,%s,%s' % (site, node,
                                              ','.join(list(map(str, parse_cmd_results(result_b))))))
df = pd.read_csv(str(io.StringIO('\n'.join(final_result))), sep=',')
df.to_excel("location/filename.xlsx", sheet_name="final_results", index=False)



# from tsdk import *
# from tsdk import nr_basic_audit_v2

if __name__ == '__main__':
    print("test")
    # copy database example
    # source = DbConfig(OdbcDrivers.SQLite, "10.10.10.10", "database1", "uid", "pwd")
    # destination = DbConfig(OdbcDrivers.MicrosoftSQLServer, "10.10.10.10", "database2", "uid", "pwd")
    # copy_data(source, destination)

    # server object example
    # s = TsdkServer()
    # s.set_configuration(OdbcDrivers.SQLite, "10.10.10.10", "Ericsson4g", "uid", "pwd", True)
    # s.save_configuration()
    # s.execute_sql("")
    # df1 = s.sql_to_dataframe("")
    # s.dataframe_to_sql(df1, "ada", index=False)

    # t = TsdkServer()
    # t.set_configuration(OdbcDrivers.SQLite, "10.10.10.10", "Ericsson5g", "uid", "pwd", True)
    # s.save_configuration()
    # s.execute_sql("")

    # sch = TsdkScheduler(jobs_df=s.sql_to_dataframe("Select * from scheudler"))
    # sch.start()

    # s.dataframe_to_sql(df1, "ada", index=False)

    # nr_basic_audit_v2()



"""
Telecom Systems Development Kit - tsdk
"""

from .dbconfig import *
from .dblogs import *
from .dbmanager import *
from .dbtasks import *
from .logs import *
from .sshcmd import *
from .sshconfig import *
from .tsdk_scheduler import *
from .tsdk_server import *

__all__ = [
    'SshConfig',  # sshconfig.py
    'DbConfig', 'OdbcDrivers', 'ConnectionString',  # dbconfig.py
    'SQLServer',  # dbmanager.py
    'copy_data',  # dbtasks.py
    'SystemSchemas', 'ExcludeNEsParams', 'ExcludeNEsSchema',  # tsdk_server.py
    'LogicsSchemaParams', 'SchedulerSchemaParams', 'LogsSchemaParams', 'LogicsLogSchemaParams',  # tsdk_server.py
    'LogicsSchema', 'SchedulerSchema', 'LogsSchema', 'LogicsLogSchema',  # tsdk_server.py
    'Logics', 'TsdkServer',  # tsdk_server.py
    'ErrorLog', 'QueryLog', 'SchedulerLog', 'Log', 'AllLog',  # logs.py
    'DBLogs', 'DBLogicsLogs',  # dblogs.py
    'TsdkScheduler'  # tsdk_scheduler.py
]



"""
Telecom Systems Development Kit - tsdk
"""

import sys
import time
from copy import deepcopy

import paramiko
import select

from tsdk.common.sshconfig import SshConfig as _SshConfig


class Ssh:
    def __init__(self,
                 sshconfig: _SshConfig):
        self._sshconfig = deepcopy(sshconfig)

    @property
    def sshconfig(self):
        """Get Ssh Configuration

        Returns:
            SshConfig: Ssh Configuration Parameters in SshConfig Class
        """
        return self._sshconfig

    @sshconfig.setter
    def sshconfig(self, sshconfig: _SshConfig) -> None:
        """Set Ssh Configuration

        Args:
            sshconfig (SshConfig): Ssh Configuration Parameters in SshConfig class
        """
        self._sshconfig = deepcopy(sshconfig)

    def run_cmd(self, host_ip, cmd_list):
        i = 0
        while True:
            print("Trying to connect to %s (%i/%i)" % (host_ip, i, self._sshconfig.retry_time))
            try:
                ssh = paramiko.SSHClient()
                ssh.load_system_host_keys()
                ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
                ssh.connect(host_ip, port=7777, username="atec", password="atecadmin")
                break
            except paramiko.AuthenticationException:
                print("Authentication failed when connecting to %s" % host_ip)
                sys.exit(1)
            except:
                print("Could not SSH to %s, waiting for it to start" % host_ip)
                i += 1
                time.sleep(2)

        # If we could not connect within time limit
        if i >= self._sshconfig.retry_time:
            print("Could not connect to %s. Giving up" % host_ip)
            sys.exit(1)
        # After connection is successful
        # Send the command
        for command in cmd_list:
            # print command
            print("> " + command)
            # execute commands
            stdin, stdout, stderr = ssh.exec_command(command)
            # TODO() : if an error is thrown, stop further rules and revert back changes
            # Wait for the command to terminate
            while not stdout.channel.exit_status_ready():
                # Only print data if there is data to read in the channel
                if stdout.channel.recv_ready():
                    rl, wl, xl = select.select([stdout.channel], [], [], 0.0)
                    if len(rl) > 0:
                        tmp = stdout.channel.recv(1024)
                        output = tmp.decode()
                        print(output)

        # Close SSH connection
        ssh.close()
        return



import logging as _logging

__all__ = ['ErrorLog', 'QueryLog', 'SchedulerLog', 'Log', 'AllLog']


class ErrorLog(object):
    @staticmethod
    def write(msg):
        _logging.getLogger('ErrorLog').error(msg)


class QueryLog(object):
    @staticmethod
    def write(msg):
        _logging.getLogger('QueryLog').error(msg)


class SchedulerLog(object):
    @staticmethod
    def write(msg):
        _logging.getLogger('apscheduler').error(msg)


class Log(object):
    @staticmethod
    def write(msg):
        _logging.getLogger('Log').error(msg)


class AllLog(object):
    @staticmethod
    def write(msg):
        _logging.getLogger('AllLog').error(msg)



"""
Telecom Systems Development Kit - tsdk
"""

import urllib.parse as _parse
from enum import Enum as _Enum

__all__ = ['OdbcDrivers',
           'DbConfig',
           'ConnectionString']


class OdbcDrivers(_Enum):
    """ODBC Drivers Enumerator
    """
    MicrosoftSQLServer = 0
    SQLite = 1


def get_odbc_driver_str(odbc_driver: OdbcDrivers) -> str:
    """Get ODBC Driver Name

    Args:
        odbc_driver (OdbcDrivers): ODBC driver from OdbcDrivers Enum

    Returns:
        str: ODBC Driver name
    """
    if odbc_driver is OdbcDrivers.SQLite:
        return "SQLite3 ODBC Driver"
    elif odbc_driver is OdbcDrivers.MicrosoftSQLServer:
        return "ODBC Driver 17 for SQL Server"
    else:
        return "ODBC Driver 17 for SQL Server"


def get_odbc_driver(driver: str) -> OdbcDrivers:
    """Get ODBC Driver Name

    Args:
        driver (str): ODBC driver String

    Returns:
        OdbcDrivers: ODBC driver from OdbcDrivers Enum
    """
    if driver is "SQLite3 ODBC Driver":
        return OdbcDrivers.SQLite
    elif driver is "ODBC Driver 17 for SQL Server":
        return OdbcDrivers.MicrosoftSQLServer
    else:
        return OdbcDrivers.MicrosoftSQLServer


class DbConfig(object):
    """Database Configuration Object

    Args:
        driver (OdbcDrivers): ODBC driver from OdbcDrivers Enum
        server (str): Server Address
        database (str): Database Name
        uid (str): User Id
        pwd (str): Password
        trusted (bool): Trusted = Yes If using local authentication

    Properties:
        All parameters are available as Get/Set properties
    """

    def __init__(self,
                 driver: OdbcDrivers = OdbcDrivers.MicrosoftSQLServer,
                 server: str = "",
                 database: str = "",
                 uid: str = "",
                 pwd: str = "",
                 trusted: bool = False) -> None:
        """
        Initialize Database Configuration
        """
        self.driver: OdbcDrivers = driver
        self.server: str = server
        self.database: str = database
        self.uid: str = uid
        self.pwd: str = pwd
        self.trusted: bool = trusted

    def parse_odbc(self, odbc: str):
        keys_by_semicolon = odbc.split(";")
        conn_string_keys = {}
        for key_semicolon in keys_by_semicolon:
            keys_by_equals = key_semicolon.split("=")
            if len(keys_by_equals) == 1:
                conn_string_keys[keys_by_equals[0].upper()] = ""
            elif len(keys_by_equals) == 2:
                conn_string_keys[keys_by_equals[0].upper()] = keys_by_equals[1]
        if "DRIVER" in conn_string_keys:
            self.driver = get_odbc_driver(conn_string_keys["DRIVER"].lstrip("{").rstrip("}"))
        if "DATA SOURCE" in conn_string_keys:
            self.server = conn_string_keys["DATA SOURCE"]
        if "DATABASE" in conn_string_keys:
            self.database = conn_string_keys["DATABASE"]
        if "INITIAL CATALOG" in conn_string_keys:
            self.database = conn_string_keys["INITIAL CATALOG"]
        if "USER ID" in conn_string_keys:
            self.uid = conn_string_keys["USER ID"]
        if "PASSWORD" in conn_string_keys:
            self.pwd = conn_string_keys["PASSWORD"]
        if "INTEGRATED SECURITY" in conn_string_keys:
            self.trusted = True


class ConnectionString(object):
    """Connection String Object

    Args:
        dbconfig (DbConfig): Server Configuration Parameters in DbConfig Structure

    Properties:
        get_odbc:
            Returns:
                str: ODBC Connection String for pyodbc

        get_url:
            Returns:
                str: DSN style Connection String for sqlalchemy Engine

    """

    def __init__(self, dbconfig: DbConfig) -> None:
        """
        Initialize Connection String
        """
        self.dbconfig = DbConfig(dbconfig.driver,
                                 dbconfig.server,
                                 dbconfig.database,
                                 dbconfig.uid,
                                 dbconfig.pwd,
                                 dbconfig.trusted)

    @property
    def get_odbc(self) -> str:
        """Get ODBC

        Returns:
            str: ODBC Connection String for pyodbc
        """
        return str("DRIVER={" + get_odbc_driver_str(self.dbconfig.driver) + "};" +
                   "SERVER=" + self.dbconfig.server + ";" +
                   "DATABASE=" + self.dbconfig.database + ";" +
                   "TRUSTED_CONNECTION=YES" if self.dbconfig.trusted else
                   "DRIVER={" + get_odbc_driver_str(self.dbconfig.driver) + "};" +
                   "SERVER=" + self.dbconfig.server + ";" +
                   "DATABASE=" + self.dbconfig.database + ";" +
                   "UID=" + self.dbconfig.uid + ";" +
                   "PWD=" + self.dbconfig.pwd + ";")

    @property
    def get_url(self) -> str:
        """Get Url

        Returns:
            str: DSN style Connection String for sqlalchemy Engine
        """
        if self.dbconfig.driver == OdbcDrivers.MicrosoftSQLServer:
            params = _parse.quote_plus(
                "DRIVER={" + get_odbc_driver_str(self.dbconfig.driver) + "};" +
                "SERVER=" + self.dbconfig.server + ";" +
                "DATABASE=" + self.dbconfig.database + ";" +
                "TRUSTED_CONNECTION=YES" if self.dbconfig.trusted else
                "DRIVER={" + get_odbc_driver_str(self.dbconfig.driver) + "};" +
                "SERVER=" + self.dbconfig.server + ";" +
                "DATABASE=" + self.dbconfig.database + ";" +
                "UID=" + self.dbconfig.uid + ";" +
                "PWD=" + self.dbconfig.pwd)
            return str("mssql+pyodbc:///?odbc_connect=%s" % params)
        else:
            return str("sqlite:///" + self.dbconfig.database)



"""
Telecom Systems Development Kit - tsdk
"""

import logging
from os.path import dirname, abspath

from tsdk.common import *
from tsdk.logics import *

__all__ = [
    'SshConfig',  # sshconfig.py
    'DbConfig', 'OdbcDrivers', 'ConnectionString',  # dbconfig.py
    'SQLServer',  # dbmanager.py
    'copy_data',  # dbtasks.py
    'SystemSchemas', 'ExcludeNEsParams', 'ExcludeNEsSchema',  # tsdk_server.py
    'LogicsSchemaParams', 'SchedulerSchemaParams', 'LogsSchemaParams', 'LogicsLogSchemaParams',  # tsdk_server.py
    'LogicsSchema', 'SchedulerSchema', 'LogsSchema', 'LogicsLogSchema',  # tsdk_server.py
    'Logics', 'TsdkServer',  # tsdk_server.py
    'ErrorLog', 'QueryLog', 'SchedulerLog', 'Log', 'AllLog',  # logs.py
    'DBLogs', 'DBLogicsLogs',  # dblogs.py
    'TsdkScheduler',  # tsdk_scheduler.py
    'nr_basic_audit_v2'
]

ROOT_DIR = dirname(abspath(__file__))

handler = logging.FileHandler('tsdk_error.log', 'a')
handler.setFormatter(logging.Formatter('%(asctime)s @ %(levelname)s @ %(funcName)s @ %(message)s'))
log = logging.getLogger('ErrorLog')
log.setLevel(logging.ERROR)
log.addHandler(handler)

handler = logging.FileHandler('tsdk_query.log', 'a')
handler.setFormatter(logging.Formatter('%(asctime)s @ %(funcName)s @ %(message)s'))
log = logging.getLogger('QueryLog')
log.setLevel(logging.ERROR)
log.addHandler(handler)

handler = logging.FileHandler('tsdk_log.log', 'a')
handler.setFormatter(logging.Formatter('%(asctime)s @ %(funcName)s @ %(message)s'))
log = logging.getLogger('Log')
log.setLevel(logging.ERROR)
log.addHandler(handler)

handler = logging.FileHandler('tsdk_all.log', 'a')
handler.setFormatter(logging.Formatter('%(asctime)s @ %(funcName)s @ %(message)s'))
log = logging.getLogger('AllLog')
log.setLevel(logging.ERROR)
log.addHandler(handler)

handler = logging.FileHandler('tsdk_scheduler.log', 'a')
handler.setFormatter(logging.Formatter('%(asctime)s @ %(funcName)s @ %(message)s'))
log = logging.getLogger('apscheduler')
log.setLevel(logging.ERROR)
log.addHandler(handler)

del handler, log, logging, dirname, abspath



"""
Telecom Systems Development Kit - tsdk
"""

__all__ = ['SshConfig']


class SshConfig(object):
    """Ssh Configuration Object

    Args:
        host (str): Host Ip/Address
        port (int): Host Port
        uid (str): User Id
        pwd (str): Password
        retry_time (int): Retry Time
    Properties:
        All parameters are available as Get/Set properties
    """

    def __init__(self,
                 host: str,
                 port: int,
                 uid: str,
                 pwd: str,
                 retry_time: int) -> None:
        """
        Initialize Ssh Configuration
        """
        self.host: str = host
        self.port: int = port
        self.uid: str = uid
        self.pwd: str = pwd
        self.retry_time: int = retry_time



"""
Telecom Systems Development Kit - tsdk
"""

import sqlite3 as _sqlite3

import pandas as _pd
import pyodbc as _po
from sqlalchemy import create_engine as _create_engine

from tsdk.common.dbconfig import ConnectionString as _ConnectionString
from tsdk.common.dbconfig import DbConfig as _DbConfig
from tsdk.common.dbconfig import OdbcDrivers as _OdbcDrivers
from tsdk.common.logs import ErrorLog as _Error_log
from tsdk.common.logs import QueryLog as _Query_log

__all__ = ['SQLServer']


class SQLServer(object):

    def __init__(self, dbconfig: _DbConfig):
        self._connectionString: \
            _ConnectionString = _ConnectionString(dbconfig)
        self._connection = None
        self._engine = None
        self.Cursor = None

    @property
    def Engine(self):
        self._engine = \
            _create_engine(self._connectionString.get_url, fast_executemany=True)
        return self._engine

    @property
    def Connection(self):
        if self._connectionString.dbconfig.driver == _OdbcDrivers.SQLite:
            self._connection = \
                _sqlite3.connect(self._connectionString.dbconfig.database)
        else:
            self._connection = _po.connect(self._connectionString.get_odbc)
        return self._connection

    @property
    def ConnectionString(self):
        return self._connectionString

    def sql_to_dataframe(self, sql: str) -> (_pd.DataFrame, Exception):
        _Query_log.write(sql)
        df = _pd.DataFrame()
        err = None
        try:
            if self._connectionString.dbconfig.driver == _OdbcDrivers.SQLite:
                self._connection = \
                    _sqlite3.connect(self._connectionString.dbconfig.database)
            else:
                self._connection = _po.connect(self._connectionString.get_odbc)
            df = _pd.read_sql(sql, con=self._connection)
            self._connection.close()
        except Exception as e:
            _Error_log.write(e)
            err = e
        return df, err

    def sql_to_dataframe_2(self, sql: str) -> (_pd.DataFrame, Exception):
        _Query_log.write(sql)
        err = None
        df = _pd.DataFrame()
        try:
            if self._connectionString.dbconfig.driver == _OdbcDrivers.SQLite:
                self._connection = \
                    _sqlite3.connect(self._connectionString.dbconfig.database)
            else:
                self._connection = _po.connect(self._connectionString.get_odbc)
            self.Cursor = self._connection.cursor()
            self.Cursor.execute(sql)
            df = _pd.DataFrame([list(row) for row in self.Cursor],
                               columns=[column[0] for column in self.Cursor.description])
            self.Cursor.close()
            self._connection.close()
        except Exception as e:
            _Error_log.write(e)
            err = e
        return df, err

    def dataframe_to_db(self,
                        df: _pd.DataFrame,
                        table_name: str,
                        if_exists='append',
                        index=False) -> Exception:
        err = None
        try:
            self._engine = \
                _create_engine(self._connectionString.get_url, fast_executemany=True)
            df.to_sql(table_name,
                      con=self._engine,
                      if_exists=if_exists,
                      index=index)
            self._engine.dispose()
        except Exception as e:
            _Error_log.write(e)
            err = e
        return err

    def execute_sql(self, sql: str) -> Exception:
        _Query_log.write(sql)
        err = None
        try:
            if self._connectionString.dbconfig.driver == _OdbcDrivers.SQLite:
                self._connection = \
                    _sqlite3.connect(self._connectionString.dbconfig.database)
            else:
                self._connection = _po.connect(self._connectionString.get_odbc)
            self.Cursor = self._connection.cursor()
            self.Cursor.execute(sql)
            self._connection.commit()
            self.Cursor.close()
            self._connection.close()
        except Exception as e:
            _Error_log.write(e)
            err = e
        return err

    def load_all_excel_sheets_to_db(self, excel_file):
        err = None
        f = _pd.ExcelFile(excel_file)
        for sheet in f.sheet_names:
            try:
                df = f.parse(sheet)
                self.dataframe_to_db(df, sheet)
            except Exception as e:
                _Error_log.write(e)
                err = e
        return err

    def load_excel_sheet_to_db(self, excel_file, sheet_name):
        err = None
        f = _pd.ExcelFile(excel_file)
        for sheet in f.sheet_names:
            if sheet == sheet_name:
                try:
                    df = f.parse(sheet)
                    self.dataframe_to_db(df, sheet)
                except Exception as e:
                    _Error_log.write(e)
                    err = e
        return err

    def load_csv_to_db(self, csv_file, table_name):
        err = None
        try:
            df = _pd.read_csv(csv_file)
            self.dataframe_to_db(df, table_name)
        except Exception as e:
            _Error_log.write(e)
            err = e
        return err



"""
Telecom Systems Development Kit - tsdk
"""

import pandas as _pd

from tsdk.common.dbconfig import DbConfig as _DbConfig
from tsdk.common.dbconfig import OdbcDrivers as _OdbcDrivers
from tsdk.common.dbmanager import SQLServer as _SQLServer
from tsdk.common.logs import ErrorLog as _Error_log

__all__ = ['copy_data']


def copy_data(source: _DbConfig, destination: _DbConfig) -> Exception:
    err = None
    try:
        source_db = _SQLServer(source)
        destination_db = _SQLServer(destination)
        sql = "SELECT TABLE_NAME AS name FROM INFORMATION_SCHEMA.TABLES" \
            if (source.driver == _OdbcDrivers.MicrosoftSQLServer) \
            else "SELECT name FROM sqlite_master WHERE type='table'"
        tables, err = source_db.sql_to_dataframe(sql)
        if len(err.__class__.__name__) > 0:
            return err
        for index, row in tables.iterrows():
            print(row["name"])
            try:
                conn = source_db.Connection
                for chunk in _pd.read_sql("SELECT * FROM " + row["name"],
                                          con=conn,
                                          chunksize=50000):
                    engine = destination_db.Engine
                    chunk.to_sql(row["name"],
                                 con=engine,
                                 if_exists='append',
                                 index=False)
                    engine.dispose()
                conn.close()
                if destination.driver == _OdbcDrivers.MicrosoftSQLServer:
                    err = destination_db.execute_sql(
                        "DROP TABLE IF EXISTS temp_name")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "SELECT DISTINCT * INTO temp_name FROM " + row["name"])
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "DROP TABLE IF EXISTS " + row["name"])
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "EXEC sp_rename temp_name, " + row["name"] + "")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                else:
                    err = destination_db.execute_sql(
                        "DROP TABLE IF EXISTS temp_name")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "CREATE TABLE temp_name AS SELECT DISTINCT * FROM " + row["name"] + ";")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "DROP TABLE IF EXISTS " + row["name"] + ";")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
                    err = destination_db.execute_sql(
                        "ALTER TABLE temp_name RENAME TO " + row["name"] + "")
                    if len(err.__class__.__name__) > 0:
                        _Error_log.write(err)
            except Exception as e:
                _Error_log.write(e)
                err = e
                continue
    except Exception as e:
        _Error_log.write(e)
        err = e
    return err



"""
Telecom Systems Development Kit - tsdk
"""

import os
import re

import paramiko
from paramiko_expect import SSHClientInteraction


def strip_ansi_codes(s):
    return re.sub(r'\x1b\[([\d,A-Z]{1,2}(;\d{1,2})?(;\d{3})?)?[m|K]?', '', s)


HOST = '10.242.66.8'
USERNAME = 'fmehmoo1'
PASSWORD = 'PassCode@987'
PORT = '5023'


class Connectivity:
    def __init__(self, HOST, USERNAME, PASSWORD, PORT, display_exec_msg):

        self.HOST = HOST
        self.USERNAME = USERNAME
        self.PASSWORD = PASSWORD
        self.PORT = PORT
        self.display_exec_msg = display_exec_msg
        self.display = False
        self.client = None
        self.interact = None
        self.oss_PROMPT = None

        self.bts_PROMPT = r'.*>\s+$'

        self.ex_script_cmds = True
        self.oss_connected = False
        self.bts_connected = False

        self.response = None
        self.all_output = []
        self.bts_alive = False

        self.gen_oss_prompt()

    def gen_oss_prompt(self):

        # generate the OSS prompt which the paramiko will check after execution of commands
        # need to adjust accordingly to server trying to connect

        if self.HOST == '10.242.66.8':
            # self.oss_PROMPT = r'.*fmehmoo1\@scp\-3\-scripting\(syleenm1\).*'
            self.oss_PROMPT = r'.*fmehmoo1.*'

    def connect_oss(self):
        try:
            self.all_output.append('Connecting to OSS...')
            self.client = paramiko.SSHClient()
            self.client.load_system_host_keys()
            self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            self.client.connect(hostname=self.HOST, port=self.PORT, username=self.USERNAME, password=self.PASSWORD,
                                look_for_keys=False)
            self.interact = SSHClientInteraction(self.client, timeout=30, display=self.display,
                                                 output_callback=self.display_exec_msg)
            self.interact.expect(self.oss_PROMPT)
            cmd_output_ls = self.interact.current_output_clean
            self.all_output.append(cmd_output_ls)
            self.oss_connected = True
        except Exception as e:
            print(str(e))
            self.oss_connected = False
            # cmd_output_ls = interact.current_output
            self.all_output.append('Failed to connect to OSS')
            # if cmd_output_ls:
            #     self.all_output.append(cmd_output_ls)
            self.all_output.append('Value:: ' + str(e))

    def disconnect_oss(self):
        print('Disconnecting OSS Now')
        if self.oss_connected:
            self.all_output.append('Disconnecting OSS...')
            self.interact.send('exit')
            # self.interact.expect()
            self.oss_connected = False
            self.client.close()

    def check_bts_connectivity(self, bts_ip):
        self.bts_alive = False
        # ping_cmd = 'ping ' + bts_ip
        ping_cmd = 'ping -c 3 ' + bts_ip
        ping_output = self.send_cmd(ping_cmd, self.oss_PROMPT)
        result = re.match(r'^.*is alive.*$', ping_output)
        regex = re.compile(r'.* ([1-9]) received, .*$', re.MULTILINE)
        matches = [m.groups() for m in regex.finditer(ping_output)]
        # print(matches)
        if matches:
            # eq_lst = [int(m[0]) for m in matches]
            eq_lst = int(matches[0][0])
            print(eq_lst)
        else:
            print('no match in ping output')
        if result or matches:
            self.bts_alive = True
        else:
            print('ping commands no response. Now Amos in bts to check')
            self.connect_bts(bts_ip)
            if self.bts_connected:
                self.bts_connected = False
                self.disconnect_bts()

    def connect_bts(self, bts_ip):
        # Checking ip contact...Not OK
        # Checking ip contact...OK
        print('Amos to BTS ', bts_ip)
        cmd = 'amos ' + bts_ip
        # Prompt = r'Checking ip contact\.\.\..*'
        Prompt = r'.*>\s+$|Unable to connect to.*'
        output = self.send_cmd(cmd, Prompt)
        if output:
            regex = re.search(r'Checking ip contact\.\.\.OK.*', output, re.MULTILINE)
            if regex:
                self.bts_alive = True
                self.bts_connected = True
                print('Amos OK')
            else:
                self.bts_alive = False
                self.bts_connected = False
                print('Amos NOK')
        else:
            self.bts_alive = False
            self.bts_connected = False
            print('Amos NOK')

    def disconnect_bts(self, ):
        print('Disconnecting BTS...')
        cmd = 'exit'
        Prompt = self.oss_PROMPT
        output = self.send_cmd(cmd, Prompt)
        self.all_output.append(output)
        self.bts_connected = False

    def send_cmd(self, cmd, PROMPT=None):
        cmd_output_ls = None
        try:
            if PROMPT is None:
                PROMPT = self.bts_PROMPT
            self.all_output.append("Command Sent <" + cmd + ">")
            # print('Command Send--->',cmd)
            if self.ex_script_cmds:
                self.interact.send(cmd)
                self.interact.expect(PROMPT)
                cmd_output_ls = strip_ansi_codes(self.interact.current_output_clean)
                print('raw ouput-->', self.interact.current_output)
                if cmd_output_ls:
                    self.all_output.append(cmd_output_ls)
                    # print(cmd_output_ls)
            return cmd_output_ls
        except Exception as ex:
            print('Exception in Send command')
            return cmd_output_ls

    @staticmethod
    def get_logfile(cmd_output_ls):
        log_file = ''
        log_file_output = strip_ansi_codes(cmd_output_ls)
        result = re.match(r'^\s*Logging to file:\s*(.*\.log)', log_file_output)
        if result:
            print("-------------\t log file found >" + result[1])
            log_file = result[1]
            # self.update_msg2.emit("Output Logs file for " + rnc + ": " + log_file)
        else:
            # self.update_msg2.emit("Output Logs file error")
            print("Log file not found")
        return log_file


def display_exec_msg(msg):
    print(msg)


class Implementation:

    def __init__(self):
        pass

    def main(self):

        self.remote_path = r'/home/shared/fmehmoo1/Scripts/'
        self.local_path = r'D:\scripts'
        self.files = ['L700_V123.txt', 'L700_V11.txt', 'L700_V122.txt', 'L700_V124.txt']
        self.files = ['L700_V123.txt', 'L700_V11.txt']
        self.bts_info = ['NJ01059A']
        # self.upload(self.remote_path, self.local_path,self.files)
        self.conectivity()

    def conectivity(self):
        # Initiate connection object
        oss_con = Connectivity(HOST, USERNAME, PASSWORD, PORT, display_exec_msg)
        # connect to oss
        oss_con.connect_oss()
        # check if connected to oss
        if oss_con.oss_connected:
            print('Connected to host')
            # cmd = 'time'
            # response = oss_con.send_cmd(cmd, oss_con.oss_PROMPT)
            # cmd = 'ls -ltr'
            # response = oss_con.send_cmd(cmd, oss_con.oss_PROMPT)
            for bts in self.bts_info:
                # cmd = r'cat /opt/ericsson/amos/moshell/sitefiles/ipdatabase | grep -i NJ01059A'
                cmd = r'cat /opt/ericsson/amos/moshell/sitefiles/ipdatabase | egrep -h NJ01059A'
                response = oss_con.send_cmd(cmd, oss_con.oss_PROMPT)
                # print(response)
                parsed_output = response.split('\n')
                parsed_output = parsed_output[1:-1]
                # print(parsed_output)
                for line in parsed_output:
                    print('line-->', line)
                    temp = line.split()
                    print('parsed_output->>', temp[0])
            output = {}
            # for bts in self.bts_info:
            #     oss_con.check_bts_connectivity(bts)
            #
            #     print(' Connectiving to bts {}'.format(bts))
            #     # if bts connectivity is ok
            #     if oss_con.bts_alive:
            #
            #         # connect to bts now
            #         oss_con.connect_bts(bts)
            #         print('connected to bts')
            #
            #         # check if connected to bts ok
            #         if oss_con.bts_connected:
            #             # sample command to send to bts after connected
            #
            #             output[str(bts)] = []
            #
            #             for script in self.files:
            #                 cmd ='run ' + os.path.join(self.remote_path,script)
            #                 print('\tImplementing script --->{}',cmd)
            #                 # response of the command --> do whatever required simeple print or save in file ...
            #                 response = oss_con.send_cmd(cmd)
            #
            #                 output[str(bts)].append(response)
            #
            #             # #------------optional -----------------
            #             # #sample command -2 . this will enable loggin on the oss side -- optiional depends on scenario
            #             # cmd = 'l+'
            #             # response = oss_con.send_cmd(cmd)
            #             # # auto generated log file on the oss , depends on if l+ command is sent or not
            #             # log_file = oss_con.get_logfile(response)
            #             # print('log_file-->', log_file)
            #             # # ------------optional -----------------
            #             #
            #             # # repeat above cmds procedure as required
            #             #
            #             #
            #             # #send command if l+ is sent before otherwise skip this
            #             # cmd = 'l-'
            #             # response = oss_con.send_cmd(cmd)
            #
            #             # disconnect the BTS once done
            #             oss_con.disconnect_bts()
            #
            #         else:
            #             print('Failed to Connect to BTS')
            #
            #     else:
            #         print('BTS {} not Reachable...'.format(bts))
            #
            #
            # dis connect the OSS
            oss_con.disconnect_oss()
            print('Respnose print')
            # print(oss_con.all_output)
            with open(os.path.join(self.local_path, 'logs.txt'), 'w') as file:
                file.writelines('\n'.join(oss_con.all_output))
        else:
            print('Failed to connect OSS')

    @staticmethod
    def upload(remote_path, local_path, files):
        transport = paramiko.Transport((HOST, int(PORT)))
        transport.connect(username=USERNAME, password=PASSWORD)
        sftp = paramiko.SFTPClient.from_transport(transport)
        import os
        if isinstance(files, list):
            for file in files:
                sftp.put(os.path.join(local_path, file), os.path.join(remote_path, file))
        else:
            sftp.put(os.path.join(local_path, files), os.path.join(remote_path, files))
            pass
        sftp.close()
        transport.close()
        print('upload done')


if __name__ == "__main__":
    imp = Implementation()
    imp.main()
    print('Completed')



import pandas as _pd

from tsdk.common.tsdk_server import LogicsLogSchema as _LogicsLogSchema
from tsdk.common.tsdk_server import LogsSchema as _LogsSchema
from tsdk.common.tsdk_server import SystemSchemas as _SystemSchemas
from tsdk.common.tsdk_server import TsdkServer as _Tsdk

__all__ = ['DBLogs', 'DBLogicsLogs']


class DBLogs(object):
    def __init__(self,
                 logicname,
                 periodicity,
                 category,
                 printerrors,
                 developer):
        self._logicname = logicname
        self._periodicity = periodicity
        self._category = category
        self._printerrors = printerrors
        self._developer = developer

    def write(self, msg):
        df = _pd.DataFrame(_LogsSchema)
        df.iloc[0] = [self._logicname,
                      self._periodicity,
                      self._category,
                      self._printerrors,
                      self._developer,
                      msg,
                      _pd.Timestamp.now(),
                      _pd.to_datetime(_pd.Timestamp.now()).date()]
        return _Tsdk().dataframe_to_db(df, _SystemSchemas.Logs)


class DBLogicsLogs(object):
    def __init__(self):
        pass

    def write(self, logicname, family, database, layertechnology, mokey, parameter, oldvalue, newvalue, time, date):
        df = _pd.DataFrame(_LogicsLogSchema)
        df.iloc[0] = [logicname,
                      family,
                      database,
                      layertechnology,
                      mokey,
                      parameter,
                      oldvalue,
                      newvalue,
                      _pd.Timestamp.now(),
                      _pd.to_datetime(_pd.Timestamp.now()).date()]
        return _Tsdk().dataframe_to_db(df, _SystemSchemas.LogicsLogs)



"""
Telecom Systems Development Kit - tsdk
"""

import json as _json

import pandas as _pd
from strenum import StrEnum as _StrEnum

import tsdk.common.dbconfig as _dbconfig
from tsdk.common.dbconfig import DbConfig as _DbConfig
from tsdk.common.dbconfig import OdbcDrivers as _OdbcDrivers
from tsdk.common.dbmanager import SQLServer as _SQLServer

__all__ = [
    'SystemSchemas',
    'LogicsSchemaParams',
    'SchedulerSchemaParams',
    'LogsSchemaParams',
    'LogicsLogSchemaParams',
    'ExcludeNEsParams',
    'LogicsSchema',
    'SchedulerSchema',
    'LogsSchema',
    'LogicsLogSchema',
    'ExcludeNEsSchema',
    'Logics',
    'TsdkServer']


class _Conf(object):
    Driver = _OdbcDrivers.MicrosoftSQLServer
    Server = "desktop-v8venfm"
    Database = "tsdk"
    Uid = ""
    Pwd = ""
    Trusted = True


class SystemSchemas(_StrEnum):
    Logics = "logics",
    Scheduler = "scheduler",
    Logs = "logs",
    LogicsLogs = "logics_logs",
    ExcludeNEs = "exclude_nes"


class ExcludeNEsParams(_StrEnum):
    MOKey = "mo_key"


class LogicsSchemaParams(_StrEnum):
    LogicName = "logic_name",
    Family = "family",
    Database = "database",
    LayerTechnology = "layer_technology",
    MOKey = "mo_key",
    Parameter = "parameter",
    ParameterValue = "parameter_value",
    ExcludedNEs = "excluded_nes",
    Enabled = "enabled",
    FreeSQL = "freesql",


class SchedulerSchemaParams(_StrEnum):
    Algorithm = "algorithm",
    Enabled = "enabled",
    ScheduleType = "schedule_type",
    Periodicity = "periodicity",
    RunTime = "runtime",
    StartTime = "start_time",
    EndTime = "end_time",
    Module = "module",
    Function = "function",
    Argument = "argument",
    Category = "category",
    Developer = "developer",
    ID = "id"


class LogsSchemaParams(_StrEnum):
    LogicName = "logic_name",
    Periodicity = "periodicity",
    Category = "category",
    PrintErrors = "print_errors",
    Developer = "developer",
    Msg = "msg",
    Time = "time",
    Date = "date",


class LogicsLogSchemaParams(_StrEnum):
    LogicName = "logic_name",
    Family = "family",
    Database = "database",
    LayerTechnology = "layer_technology",
    MOKey = "mo_key",
    Parameter = "parameter",
    OldValue = "old_value",
    NewValue = "new_value",
    Time = "time",
    Date = "date",


ExcludeNEsSchema = {
    ExcludeNEsParams.MOKey: _pd.Series(dtype='str')
}

LogicsSchema = {
    LogicsSchemaParams.LogicName: _pd.Series(dtype='str'),
    LogicsSchemaParams.Family: _pd.Series(dtype='str'),
    LogicsSchemaParams.Database: _pd.Series(dtype='str'),
    LogicsSchemaParams.LayerTechnology: _pd.Series(dtype='str'),
    LogicsSchemaParams.MOKey: _pd.Series(dtype='str'),
    LogicsSchemaParams.Parameter: _pd.Series(dtype='str'),
    LogicsSchemaParams.ParameterValue: _pd.Series(dtype='str'),
    LogicsSchemaParams.ExcludedNEs: _pd.Series(dtype='str'),
    LogicsSchemaParams.Enabled: _pd.Series(dtype='bool'),
    LogicsSchemaParams.FreeSQL: _pd.Series(dtype='str')
}

SchedulerSchema = {
    SchedulerSchemaParams.Algorithm: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Enabled: _pd.Series(dtype='bool'),
    SchedulerSchemaParams.Periodicity: _pd.Series(dtype='int'),
    SchedulerSchemaParams.RunTime: _pd.Series(dtype='str'),
    SchedulerSchemaParams.StartTime: _pd.Series(dtype='str'),
    SchedulerSchemaParams.EndTime: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Module: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Function: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Argument: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Category: _pd.Series(dtype='str'),
    SchedulerSchemaParams.Developer: _pd.Series(dtype='str'),
    SchedulerSchemaParams.ID: _pd.Series(dtype='str')
}

LogsSchema = {
    LogsSchemaParams.LogicName: _pd.Series(dtype='str'),
    LogsSchemaParams.Periodicity: _pd.Series(dtype='int'),
    LogsSchemaParams.Category: _pd.Series(dtype='str'),
    LogsSchemaParams.PrintErrors: _pd.Series(dtype='str'),
    LogsSchemaParams.Developer: _pd.Series(dtype='str'),
    LogsSchemaParams.Msg: _pd.Series(dtype='str'),
    LogsSchemaParams.Time: _pd.Series(dtype='str'),
    LogsSchemaParams.Date: _pd.Series(dtype='str')
}

LogicsLogSchema = {
    LogicsLogSchemaParams.LogicName: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.Family: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.Database: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.LayerTechnology: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.MOKey: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.Parameter: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.OldValue: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.NewValue: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.Time: _pd.Series(dtype='str'),
    LogicsLogSchemaParams.Date: _pd.Series(dtype='str')
}


class Logics(_StrEnum):
    NR_Basic_Audit_V2 = "nr_basic_audit_v2"


class TsdkServer(_SQLServer):
    def __init__(self,
                 dbconfig: _DbConfig = _DbConfig(_Conf.Driver,
                                                 _Conf.Server,
                                                 _Conf.Database,
                                                 _Conf.Uid,
                                                 _Conf.Pwd,
                                                 _Conf.Trusted)) -> None:
        super().__init__(dbconfig)
        self.conf = 'conf.json'
        self.dbconfig = self.ConnectionString.dbconfig

    def set_configuration(self, Driver, Server, Database, Uid, Pwd, Trusted):
        super().__init__(_DbConfig(_dbconfig.get_odbc_driver(Driver),
                                   Server,
                                   Database,
                                   Uid,
                                   Pwd,
                                   Trusted))

    def load_configuration(self, conf: str = ''):
        if not conf:
            conf = self.conf
        with open(conf, 'rb') as f:
            conf_json = _json.load(f)
            super().__init__(_DbConfig(_dbconfig.get_odbc_driver(conf_json['DRIVER']),
                                       conf_json['SERVER'],
                                       conf_json['DATABASE'],
                                       conf_json['UID'],
                                       conf_json['PWD'],
                                       conf_json['TRUSTED']))

    def save_configuration(self, conf: str = ''):
        conf_json = {
            'DRIVER': _dbconfig.get_odbc_driver_str(self.dbconfig.server),
            'SERVER': self.dbconfig.server,
            'DATABASE': self.dbconfig.database,
            'UID': self.dbconfig.uid,
            'PWD': self.dbconfig.pwd,
            'TRUSTED': self.dbconfig.trusted,
        }
        if not conf:
            conf = self.conf
        with open(conf, 'w') as f:
            _json.dump(conf_json, f, indent=4)

    def setup_database(self):
        db = 'tsdk'
        if self.dbconfig.database != 'tsdk':
            db = self.dbconfig.database
            self.dbconfig.database = 'tsdk'

        self.execute_sql("DROP TABLE IF EXISTS " + SystemSchemas.Logics)
        df = _pd.DataFrame(LogicsSchema)
        self.dataframe_to_db(df, SystemSchemas.Logics)

        self.execute_sql("DROP TABLE IF EXISTS " + SystemSchemas.Scheduler)
        df = _pd.DataFrame(SchedulerSchema)
        self.dataframe_to_db(df, SystemSchemas.Scheduler)

        self.execute_sql("DROP TABLE IF EXISTS " + SystemSchemas.Logs)
        df = _pd.DataFrame(LogsSchema)
        self.dataframe_to_db(df, SystemSchemas.Logs)

        self.execute_sql("DROP TABLE IF EXISTS " + SystemSchemas.LogicsLogs)
        df = _pd.DataFrame(LogicsLogSchema)
        self.dataframe_to_db(df, SystemSchemas.LogicsLogs)

        self.execute_sql("DROP TABLE IF EXISTS " + SystemSchemas.ExcludeNEs)
        df = _pd.DataFrame(ExcludeNEsSchema)
        self.dataframe_to_db(df, SystemSchemas.ExcludeNEs)

        self.dbconfig.database = db



from setuptools import setup, find_packages

setup(
    name='tsdk-git',
    version='5.3.2',
    author='Ali Raza Anis',
    author_email='alirazaanis@gmail.com',
    description='Telecom System Development Kit',
    url='https://github.com/alirazaanis/tsdk-git',
    license='ATEC',
    packages=find_packages(include=['tsdk', 'tsdk.*']),
    install_requires=[
        'pandas>=1.5.1',
        'python-dateutil>=2.8.2',
        'pyodbc>=4.0.34',
        'SQLAlchemy>=1.4.42',
        'StrEnum>=0.1.0',
        'numpy>=1.23.4',
        'paramiko>=2.0.2',
        'paramiko-expect>=0.3.4',
        'APScheduler>=3.9.1',
        'openpyxl>=3.0.10'
    ]
)



"""
Telecom Systems Development Kit - tsdk
"""

import json as _json
import sys as _sys
from datetime import datetime as _datetime
from pathlib import Path as _Path

from apscheduler.events import EVENT_ALL as _EVENT_ALL
from apscheduler.executors.pool import ProcessPoolExecutor as _ProcessPoolExecutor
from apscheduler.executors.pool import ThreadPoolExecutor as _ThreadPoolExecutor
from apscheduler.jobstores.memory import MemoryJobStore as _MemoryJobStore
from apscheduler.schedulers.background import BackgroundScheduler as _BackgroundScheduler
from apscheduler.schedulers.blocking import BlockingScheduler as _BlockingScheduler
from dateutil import tz as _tz

from tsdk.common.logs import ErrorLog as _Error_log
from tsdk.common.logs import SchedulerLog as _SchedulerLog
from tsdk.common.tsdk_server import SchedulerSchemaParams as _Ssp
from tsdk.common.tsdk_server import SystemSchemas as _Ss
from tsdk.common.tsdk_server import TsdkServer as _Tsdk

__all__ = ['TsdkScheduler']

job_stores = {
    'default': _MemoryJobStore()
}
executors = {
    'default': _ThreadPoolExecutor(max_workers=60),
    'processpool': _ProcessPoolExecutor(max_workers=60)
}
job_defaults = {
    'coalesce': True,
    'max_instances': 1
}


def _listener(event):
    if hasattr(event, 'job_id'):
        if event.job_id != 'scheduler_job':
            _SchedulerLog.write('job event jobid: %s, event: %s' %
                                (event.job_id,
                                 event.exception if hasattr(event, 'exception') else ''))


def _toUTC(time_string):
    try:
        utc_zone = _tz.tzutc()
        local_zone = _tz.tzlocal()
        fmt = '%Y-%m-%d %H:%M:%S'
        local_time = _datetime.strptime(time_string, fmt)
        local_time = local_time.replace(tzinfo=local_zone)
        utc_time = local_time.astimezone(utc_zone)
        return utc_time.strftime(fmt)
    except Exception as e:
        _Error_log.write(e)
        return ''


def _validate_time(date_text):
    try:
        fmt = '%Y-%m-%d %H:%M:%S'
        _datetime.strptime(date_text, fmt)
        return True
    except ValueError:
        # print('Incorrect date format')
        return False


class TsdkScheduler(object):
    def __init__(self, scheduler_type='Blocking', jobs_df=None):
        self.type = scheduler_type
        self.job_interval = 1
        self.jobs = jobs_df
        if self.type == 'Blocking':
            self.scheduler = _BlockingScheduler(jobstores=job_stores,
                                                executors=executors,
                                                job_defaults=job_defaults)
        else:
            self.scheduler = _BackgroundScheduler(jobstores=job_stores,
                                                  executors=executors,
                                                  job_defaults=job_defaults)

    def shutdown(self):
        self.scheduler.shutdown(wait=False)

    def pause(self):
        self.scheduler.pause()

    def resume(self):
        self.scheduler.resume()

    def start(self, paused=False):
        self.scheduler.add_listener(_listener, _EVENT_ALL)
        self.scheduler.add_job(self.scheduler_job, 'interval',
                               seconds=self.job_interval, id='scheduler_job')
        self.scheduler.start(paused)

    def scheduler_job(self):
        if self.jobs is None:
            tsdk = _Tsdk()
            df, err = tsdk.sql_to_dataframe(
                "SELECT * FROM " + _Ss.Scheduler)
            df.columns = map(str.lower, df.columns)
            df = df.astype(str).apply(lambda x: x.str.lower())
        else:
            df = self.jobs
        # for job in self.scheduler.get_jobs():
        #    print("name: %s, trigger: %s, next run: %s, id: %s" % (
        #        job.name, job.trigger, job.next_run_time, job.id))
        for index, row in df.iterrows():
            if row[_Ssp.Enabled]:
                if row[_Ssp.ScheduleType] == 'date':
                    if self.scheduler.get_job(row[_Ssp.ID]) is None:
                        try:
                            _sys.path.append(str(_Path(row[_Ssp.Module]).parent))
                            func = getattr(__import__(_Path(row[_Ssp.Module]).stem), row[_Ssp.Function])
                            kwargs = _json.loads(row[_Ssp.Argument].replace("'", "\"")) if len(
                                row[_Ssp.Argument]) > 0 else ''
                            self.scheduler.add_job(func, 'date',
                                                   kwargs=kwargs,
                                                   run_date=row[_Ssp.RunTime],
                                                   id=row[_Ssp.ID])
                            # print('job added %s', {row[_Ssp.ID]})
                            _SchedulerLog.write('job added %s' % row[_Ssp.ID])
                        except Exception as e:
                            _Error_log.write(e)
                elif row[_Ssp.ScheduleType] == "interval":
                    if self.scheduler.get_job(row[_Ssp.ID]) is None:
                        try:
                            _sys.path.append(str(_Path(row[_Ssp.Module]).parent))
                            func = getattr(__import__(_Path(row[_Ssp.Module]).stem), row[_Ssp.Function])
                            kwargs = _json.loads(row[_Ssp.Argument].replace("'", "\"")) if len(
                                row[_Ssp.Argument]) > 0 else ''
                            self.scheduler.add_job(func, 'interval',
                                                   kwargs=kwargs,
                                                   minutes=row[_Ssp.Periodicity],
                                                   start_date=row[_Ssp.StartTime],
                                                   end_date=row[_Ssp.EndTime],
                                                   id=row[_Ssp.ID])
                            # print('job added %s', {row[_Ssp.ID]})
                            _SchedulerLog.write('job added %s' % row[_Ssp.ID])
                        except Exception as e:
                            _Error_log.write(e)
                else:
                    pass


if __name__ == '__main__':
    import pandas as pd

    jobs = {'algorithm': ['test1', 'test2'],
            'enabled': [True, True],
            'schedule_type': ['date', 'interval'],
            'periodicity': [1440, 1440],
            'runtime': ['2023-05-17 00:00:00', ''],
            'start_time': ['', '2022-05-18 00:00:00'],
            'end_time': ['', '2023-05-17 00:00:00'],
            'module': ['c:/users/ali/desktop/test_desktop', 'c:/users/ali/desktop/test_desktop2'],
            'function': ['testx', 'testx2'],
            'argument': ['{}', ''],
            'category': ['test', 'test'],
            'developer': ['test', 'test'],
            'id': ['job1', 'job2']}

    sch = TsdkScheduler(jobs_df=pd.DataFrame.from_dict(jobs))
    sch.start()



from tsdk import Logics as _Logics
from tsdk import LogicsLogSchemaParams as _Llsp
from tsdk import LogicsSchemaParams as _Lsp
from tsdk import SystemSchemas as _Ss
from tsdk import TsdkServer as _Tsdk

__all__ = ['nr_basic_audit_v2']


def nr_basic_audit_v2():
    tsdk = _Tsdk()
    df, err = tsdk.sql_to_dataframe(
        "SELECT * FROM " + _Ss.Logics +
        " WHERE LOWER(" + _Lsp.LogicName +
        ") Like '%" + _Logics.NR_Basic_Audit_V2 +
        "%'")
    df.columns = map(str.lower, df.columns)
    df = df.astype(str)
    for index, row in df.iterrows():
        if str(row[_Lsp.Enabled]).lower() == "true":
            # family = "(SELECT T1.* FROM " + row[_LogicParams.Family] + " T1 " + \
            #         "INNER JOIN (" + \
            #         "SELECT NodeId, MAX(CAST(DateList_id AS bigint)) DateList_id FROM " + \
            #         row[_LogicParams.Family] + " GROUP BY NodeId" + \
            #         ") T2 ON " + \
            #         "T2.NodeId = T1.NodeId AND " + \
            #         "T1.DateList_id = T1.DateList_id)"
            family = row[_Lsp.Family]

            if str(row[_Lsp.ParameterValue]).lower() == "true":
                check = "(LOWER(CAST(" + row[_Lsp.Parameter] + " AS VARCHAR)) " + \
                        "<> " + \
                        "'" + str(row[_Lsp.ParameterValue]).lower() + "' AND " + \
                        "LOWER(CAST(" + row[_Lsp.Parameter] + " AS VARCHAR)) " + \
                        "<> " + \
                        "'1')"
            elif str(row[_Lsp.ParameterValue]).lower() == "false":
                check = "(LOWER(CAST(" + row[_Lsp.Parameter] + " AS VARCHAR)) " + \
                        "<> " + \
                        "'" + str(row[_Lsp.ParameterValue]).lower() + "' AND " + \
                        "LOWER(CAST(" + row[_Lsp.Parameter] + " AS VARCHAR)) " + \
                        "<> " + \
                        "'0')"
            else:
                check = "(LOWER(CAST(" + row[_Lsp.Parameter] + " AS VARCHAR)) " + \
                        "<> " + \
                        "'" + str(row[_Lsp.ParameterValue]).lower() + "')"

            sql = "SELECT " \
                  "'" + row[_Lsp.LogicName] + "' AS " + _Llsp.LogicName + ", " + \
                  "'" + row[_Lsp.Family] + "' AS " + _Llsp.Family + ", " + \
                  "'" + row[_Lsp.Database] + "' AS " + _Llsp.Database + ", " + \
                  "'" + row[_Lsp.LayerTechnology] + "' AS " + _Llsp.LayerTechnology + ", " + \
                  row[_Lsp.MOKey] + " AS " + _Llsp.MOKey + ", " + \
                  "'" + row[_Lsp.Parameter] + "' AS " + _Llsp.Parameter + ", " + \
                  row[_Lsp.Parameter] + " AS " + _Llsp.OldValue + ", " \
                                                                  "'" + row[
                      _Lsp.ParameterValue] + "' AS " + _Llsp.NewValue + ", " + \
                  " GETDATE() AS " + _Llsp.Time + ", " + \
                  " CAST( GETDATE() AS Date ) AS " + _Llsp.Date + " " + \
                  " FROM " + family + \
                  " T3 WHERE " + check

            if str(row[_Lsp.ExcludedNEs]).lower() == "yes":
                sql += " AND " + _Lsp.MOKey + \
                       " NOT IN (SELECT * FROM tsdk.dbo." + _Ss.ExcludeNEs + ")"

            if str(row[_Lsp.FreeSQL]).lower() != "nan" and \
                    str(row[_Lsp.FreeSQL]).lower() != "none":
                sql += " AND " + _Lsp.MOKey + " IN (SELECT " + _Lsp.MOKey + " FROM (" + \
                       row[_Lsp.FreeSQL] + ") T)"

            sqldb = _Tsdk()
            sqldb.ConnectionString.dbconfig.database = row[_Lsp.Database]

            df2, err = sqldb.sql_to_dataframe(sql)

            err = tsdk.dataframe_to_db(df2, _Ss.LogicsLogs)

    return err
